package Netpkg; 

use strict;
use POSIX;
use LWP::UserAgent ();
use LWP::MediaTypes qw(guess_media_type media_suffix);
use URI ();
use HTTP::Date ();
use Locale::gettext;
use Encode;
use Unicode::Normalize; #for accent-insensitive sort

use threads;
use threads::shared;
my $semaphorus : shared = 0;

# Internationalization
setlocale(LC_ALL, "");
textdomain("netpkg");

####################################################################
sub new { 
	my $self = {};
	bless ($self);	
	return $self;
}

####################################################################
# $Netpkg->GetMirrors($Hash,$configfile );
sub GetMirrors {
	my $self = shift; 
	my ($Hash,$configfile) = @_ ;
	
    open(FH, "<$configfile");
    
    while(<FH>) {
		chomp($_);    	
		if ( $_ =~ /(\s*Internet_mirror\s*=\s*)(.*)/ ){   
			next if ( $_ =~ /^[ \t]*$/ ) ;
			$Hash->{$2} = 1 ;
		} 			
	}
	close(FH);	
	
	open(FH, "<$self->{Netpkg_dir}/db/last_mirror");
	my $lastmirror;
	while(<FH>) {
		chomp($_);    
		next if ( $_ =~ /^[ \t]*$/ ) ;	
		$Hash->{$_} = 1 ;
		$lastmirror = $_ ;
	}
	$Hash->{"$lastmirror"} = 0 ;
	close(FH);	
	
	return 0; 
}

####################################################################
# $Netpkg->SaveMirrors($Hash);
sub SaveMirrors {
	my $self = shift; 
	my $Hash = shift; 
	open(FH, ">$self->{Netpkg_dir}/db/last_mirror");
	for $_ ( sort { $Hash->{$a} cmp $Hash->{$b} } keys %{$Hash} ) { 
		print FH "$_\n";
	}
	
	close(FH);	
	
	return 0; 
}

####################################################################
# $Netpkg->GetFilters();
sub GetFilters {
	my $self = shift; 
	
    open(FH, "<$self->{Netpkg_dir}/db/last_filter") or return -1 ;
    binmode FH;	
	my $Filter = <FH> ;
	chomp($Filter);    
	close(FH);
	
	return $Filter;
}

####################################################################
# $Netpkg->SaveFilters($Filter);
sub SaveFilters {
	my $self = shift; 
	my $Filter = shift; 
	open(FH, ">$self->{Netpkg_dir}/db/last_filter");
	binmode FH;	
	print FH "$Filter";
	close(FH);	
	
	return 0; 
}

####################################################################
# $Netpkg->Configure($option, $default, $configfile);
sub Configure {
	my $self = shift; 
	
	# default value
	$self->{"$_[0]"} = $_[1];
	
    open(FH, "<$_[2]");
    while (<FH>) {	
		chomp($_);

		if ( $_ =~ /^\s*${_[0]}\s*=\s*(.*)$/ ){
			$self->{"$_[0]"} = $1;
			last;
		}
	}	
	close(FH);
	return 0; 
}

####################################################################
# $Netpkg->Get ($Url, $Object, $Mode, $Data, $File, $ProgressBar, $Start, $Range);
sub Get {
	my $self = shift;
	my ($Url, $Object, $Mode, $Data, $File, $ProgressBar, $Start, $Range) = @_; 
	
	# Create an LWP UserAgent
	my $Browser = LWP::UserAgent->new;

	# Setup authentication details, if necessary
	if($self->{Proxy_Socket} ne '') {
		my $ProxyUrl = "http://"."$self->{Proxy_User}".":"."$self->{Proxy_Password}"."@"."$self->{Proxy_Socket}";
		$Browser->proxy(['http', 'ftp'], $ProxyUrl);
	}
	# if ($Browser->is_protocol_supported("ftp")) {print "ftp ok\n"}
	my $result = $Browser->head($Url);
	if ( ! $result->is_success ) {
		return -1 ;
	}
	
	$ProgressBar->set_text(decode('utf8',gettext("Loading"))." ".$Object);
	my $remote_headers = $result->headers;
	my $total_size = $remote_headers->content_length;
	my $KBtotal_size = int($total_size / 1024) ;
	my $iterate = 0 ;
	if($Mode) {
		$Browser->timeout(20);
		open(FH, ">$File") or die "Cannot open file $File : $!\n";
		binmode FH;	
		my $size;	
		my $res = $Browser->request(HTTP::Request->new(GET => $Url), sub {
				my ($chunk, $res) = @_;
				print FH $chunk;
				$size += length($chunk);
				$iterate++ ;
				if ( $iterate % 20 == 0 ){
					# print "$iterate ";
					my $Progress = $Start + $Range * $size / $total_size ;
					my $percent = int($size / $total_size * 100) ;
					if ($Progress >= 1) { $Progress = 1 }
					$ProgressBar->set_fraction($Progress);
					$ProgressBar->set_text(decode('utf8',gettext("Loading"))." ".$Object." (".$KBtotal_size." kB) ".$percent." %");
					Gtk2->main_iteration while ( Gtk2->events_pending );  
				}			
			});		
		close FH;
	}else{
		$Browser->timeout(20);
		my $res = $Browser->request(HTTP::Request->new(GET => $Url), sub {
				my($chunk, $res) = @_;
				${$Data} .= $chunk;
				$iterate++ ;
				if ( $iterate % 10 == 0 ){	
					# print "$iterate ";			
					my $len= length(${$Data});
					my $Progress = $Start + $Range * ($len / $total_size) ;
					my $percent = int(length(${$Data}) / $total_size * 100) ;
					if ($Progress >= 1) { $Progress = 1 }
					$ProgressBar->set_fraction($Progress);
					$ProgressBar->set_text(decode('utf8',gettext("Loading"))." ".$Object." (".$KBtotal_size." kB) ".$percent." %");
					Gtk2->main_iteration while ( Gtk2->events_pending );  	
				}		
			});
	}
	return 0;	
}	

####################################################################
# $Netpkg->ProcessDeps(\%db);
sub ProcessDeps {
	my $self = shift; 
	my %db = %{$_[0]};
	my $soft; 
	for $soft ( keys %db ) { 
		@{$db{$soft}}[14] = 0 ;
	}
	for $soft ( keys %db ) {
		if ( @{$db{$soft}}[13] ) {
			if ( defined @{$db{$soft}}[1] ) {
				
				# array of deps
				my @Deps = split (",", @{$db{$soft}}[5]);
				
				my $dep; 
				foreach $dep (@Deps) {
					if ( @{$db{$dep}}[12] != 0 ){
						# print "$dep\n";
						@{$db{$dep}}[14] = 1 ; 
						
						# one more level
						my @Deps2 = split (",", @{$db{$dep}}[5]);					
						my $dep2; 
						foreach $dep2 (@Deps2) {
							if ( @{$db{$dep2}}[12] != 0 ){
								# print "$dep2\n";
								@{$db{$dep2}}[14] = 1 ; 	
							}					
						}
					}
				}
			}
		}
	}
}

# Called inside a thread to execute a system command
# the shared semaphor is set to '1' when system processing ends
####################################################################
sub Execute {
	my $command = shift;
	$semaphorus = 0;	
	`$command`;
    if ($? == -1) {
    	$semaphorus = 1;
		return -1; 
	} else {
		$semaphorus = 1;
		return 0; 
	} 
}

####################################################################
# $Netpkg->Install( $File );
sub Install {
	my $self = shift; 
	my $package = shift;
	my $command = "/sbin/installpkg $package >> $self->{Logfile}";
    #print "/sbin/installpkg $package\n";
    
    # initialize the shared semaphor to '0', then wait for the thread to set it to '1'
    $semaphorus = 0;
	my $thr = threads->new(\&Execute, $command);
	while ($semaphorus == 0) {
		Gtk2->main_iteration while ( Gtk2->events_pending  );  
	} 

	return 0;
}

####################################################################
# $Netpkg->Update( $File );
sub Update {
	my $self = shift; 
	my $package = shift;
	my $command = "/sbin/upgradepkg $package >> $self->{Logfile}";
    #print "/sbin/upgradepkg --reinstall $package\n";
    
    # initialize the shared semaphor to '0', then wait for the thread to set it to '1'
    $semaphorus = 0;
	my $thr = threads->new(\&Execute, $command);
	while ($semaphorus == 0) {
		Gtk2->main_iteration while ( Gtk2->events_pending  );  
	}
	return 0;
}

####################################################################
# $Netpkg->Reinstall( $File );
sub Reinstall {
	my $self = shift; 
	my $package = shift;
	my $command = "/sbin/upgradepkg --reinstall $package >> $self->{Logfile}";
    #print "/sbin/upgradepkg --reinstall $package\n";
    
    # initialize the shared semaphor to '0', then wait for the thread to set it to '1'
    $semaphorus = 0;
	my $thr = threads->new(\&Execute, $command);
	while ($semaphorus == 0) {
		Gtk2->main_iteration while ( Gtk2->events_pending  );  
	}
	return 0;
}

####################################################################
# $Netpkg->Remove( $soft );
sub Remove {
	my $self = shift; 
	my $package = shift;
	my $command = "/sbin/removepkg $package >> $self->{Logfile}";
    #print "/sbin/removepkg $package\n";
    
    # initialize the shared semaphor to '0', then wait for the thread to set it to '1'
    $semaphorus = 0;
	my $thr = threads->new(\&Execute, $command);
	while ($semaphorus == 0) {
		Gtk2->main_iteration while ( Gtk2->events_pending  );  
	}
	return 0;
}

1;

